/***************************************************************************************************
	© 2014 McGladrey LLP  All rights reserved.

McGladrey LLP (McGladrey) grants to Client a limited, non-exclusive, non-transferable right to
install and use the Software/Source Code solely for internal business purposes.  Client may not
distribute, give, rent, loan, or resell a copy of the Software/Source Code to any third party.
McGladrey is the owner of the copyright for the Software/Source Code and McGladrey reserves all
rights related to such ownership.  Any Warranty granted by McGladrey is void if Software/Source Code
is modified by anyone other than McGladrey.

$URL: https://svn.mcgladreycloud.com/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 7851 $
$Date: 2014-02-03 15:48:42 -0600 (Mon, 03 Feb 2014) $
***************************************************************************************************/
-- Creates SQL objects used by Customer Service Alerts

set nocount on

print 'Creating table tcsa_AlertDefinitions'
go

if object_id('tcsa_AlertDefinitions') is not NULL
begin
	if (select rows from sysindexes where id = object_id('tcsa_AlertDefinitions') and indid < 2) > 0
	begin
		exec sp_rename tcsa_AlertDefinitions, tcsa_AlertDefinitions_old
	end
	else
	begin
		drop table tcsa_AlertDefinitions
	end
end
go
if object_id('pkcsa_AlertDefinitions') is not NULL
begin
	alter table tcsa_AlertDefinitions_old
		drop constraint pkcsa_AlertDefinitions
end
go
if object_id('tcsa_AlertDefinitions_old') is not NULL
begin
	if not exists (select 1 from syscolumns where object_name(id) = 'tcsa_AlertDefinitions_old' and name = 'RuleGroup')
	begin
		alter table tcsa_AlertDefinitions_old
			add RuleGroup varchar(100) not NULL default ''
	end
end
go

if object_id('tcsa_status_rules') is not NULL
begin
	if (select rows from sysindexes where id = object_id('tcsa_status_rules') and indid < 2) > 0
	begin
		exec sp_rename tcsa_status_rules, tcsa_status_rules_old
	end
	else
	begin
		drop table tcsa_status_rules
	end
end
go
if object_id('pkcsa_status_rules') is not NULL
begin
	alter table tcsa_status_rules_old
		drop constraint pkcsa_status_rules
end
go
if object_id('tcsa_status_rules_old') is not NULL
begin
	if not exists (select 1 from syscolumns where object_name(id) = 'tcsa_status_rules_old' and name = 'RuleGroup')
	begin
		alter table tcsa_status_rules_old
			add RuleGroup varchar(100) not NULL default ''
	end
end
go

create table tcsa_AlertDefinitions
(
	SEQN int identity
		constraint pkcsa_AlertDefinitions primary key clustered,
	CRITERIA varchar(3072) not NULL default '',
	ENABLED bit not NULL default 1,
	DESCRIPTION varchar(255) not NULL default '',
	ALERT_TYPE varchar(10) not NULL default 'SQL',
	ALERT_ID varchar(20) not NULL default '',
	RuleGroup varchar(100) not NULL default ''
)
go
grant select, insert, update, delete on tcsa_AlertDefinitions to iMIS
go

if object_id('tcsa_status_rules_old') is not NULL
begin
	declare @oldrows int,
		@newrows int

	insert tcsa_AlertDefinitions (CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup)
		select CRITERIA, ENABLED, '', 'SQL', ALERT_ID, RuleGroup
		from tcsa_status_rules_old
		order by SEQN

	set @newrows = @@rowcount

	select @oldrows = count(*)
		from tcsa_status_rules_old

	if @oldrows = @newrows
	begin
		drop table tcsa_status_rules_old
	end
	else
	begin
		print '!!! Error copying data from tcsa_status_rules_old into new table'
	end
end
go
if object_id('tcsa_AlertDefinitions_old') is not NULL
begin
	declare @oldrows int,
		@newrows int

	insert tcsa_AlertDefinitions (CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup)
		select CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup
		from tcsa_AlertDefinitions_old
		order by SEQN

	set @newrows = @@rowcount

	select @oldrows = count(*)
		from tcsa_AlertDefinitions_old

	if @oldrows = @newrows
	begin
		drop table tcsa_AlertDefinitions_old
	end
	else
	begin
		print '!!! Error copying data from tcsa_AlertDefinitions_old into new table'
	end
end
go
exec sp_recompile 'tcsa_AlertDefinitions'
go

print 'Creating table tcsa_AlertOutput (used by Xtender)'
go
if object_id('tcsa_Status_Output') is not NULL
begin
	-- Always ok to delete
	drop table tcsa_Status_Output
end
go
if object_id('tcsa_AlertOutput') is not NULL
begin
	-- Always ok to delete
	drop table tcsa_AlertOutput
end
go
create table tcsa_AlertOutput
(
	SEQN int identity
		constraint pkcsa_AlertOutput primary key clustered,
	OWNER_SPID smallint not NULL default @@spid,
	IMIS_USER varchar(50) not NULL default user_name(),
	EVENT_TYPE varchar(15) not NULL default 'ID',
	KEY_VALUE varchar(31) not NULL default '',
	RANK smallint not NULL default 0,						-- main window
	ICON smallint not NULL default 4,						-- detail
	KEYWORD varchar(40) not NULL default '',				-- detail
	DESCRIPTION varchar(4000) not NULL default '',			-- detail
	TOOLTIP_TEXT varchar(1000) not NULL default '',			-- detail
	TITLE_TEXT varchar(255) not NULL default '',			-- detail
	CLIPBOARD_TEXT text NOT NULL default '',				-- detail
	CLIPBOARD_DESCRIPTION varchar(100) not NULL default '',	-- detail
	ACTION_TEXT varchar(1000) not NULL default '',			-- detail
	ACTION_DESCRIPTION varchar(100) not NULL default '',	-- detail
	ACTION_TYPE varchar(30) not NULL default 'Launch',		-- detail
	LOG_TEXT varchar(1000) not NULL default '',				-- main window
	PLAY_SOUND bit not NULL default 0,						-- main window
	AUTO_POPUP bit not NULL default 0,						-- detail
	AUTO_CLIPBOARD bit not NULL default 0,					-- detail
	AUTO_ACTION bit not NULL default 0						-- detail
)
go
create index icsa_AlertOutputUSERRANK
	on tcsa_AlertOutput(OWNER_SPID, RANK)
go
grant select, insert, update, delete on tcsa_AlertOutput to iMIS
go


print 'Creating procedure pcsa_WriteSystemParams (used by Xtender)'
go
if object_id('pcsa_WriteSystemParams') is not NULL
begin
	drop procedure pcsa_WriteSystemParams
end
go
create procedure pcsa_WriteSystemParams
	@sParameterName varchar(50),
	@sShortValue varchar(255)
as
begin
	set nocount on

	if @sParameterName is not NULL
	begin
		if @sShortValue is NULL
		begin
			delete System_Params
				where ParameterName = @sParameterName
		end
		else
		begin
			insert System_Params (ParameterName, ShortValue)
				select @sParameterName, @sShortValue
				where not exists (select * from System_Params where ParameterName = @sParameterName)

			if @@rowcount = 0
			begin
				update sp
					set ShortValue = @sShortValue
					from System_Params sp
					where sp.ParameterName = @sParameterName
			end
		end
	end
end
go
grant execute on pcsa_WriteSystemParams to IMIS
go



print 'Dropping obsolete procedure pcsa_calculate_status'
go
if object_id('pcsa_calculate_status') is not NULL
begin
	drop procedure pcsa_calculate_status
end
go

print 'Adding security keyword for CSA Editor window (used by Xtender)'
go
if not exists (select * from Gen_Tables where TABLE_NAME = 'ACCESS_KEYWORDS'
		and upper(CODE) = 'CSAEDIT')
begin
	insert Gen_Tables (TABLE_NAME, CODE, UPPER_CODE, DESCRIPTION)
		values ('ACCESS_KEYWORDS', 'CSAEdit', 'CSAEDIT', 'CSA Alert Editor')
end
go

print 'Adding CSAEdit keyword to Manager, Administrator logins'
go
-- One row at a time to be as safe as possible with triggers
update Users
	set AccessKeywords = AccessKeywords + ',CSAEdit'
	where UserId = 'MANAGER'
		and upper(AccessKeywords) not like '%CSAEDIT%'
		and len(AccessKeywords) + 8 < 255	-- Only if it will fit
update Users
	set AccessKeywords = AccessKeywords + ',CSAEdit'
	where UserId = 'ADMINISTRATOR'
		and upper(AccessKeywords) not like '%CSAEDIT%'
		and len(AccessKeywords) + 8 < 255	-- Only if it will fit
go

print 'Creating table of TaskCentre servers, logins, passwords (used by Xtender)'
go
if not exists (select * from Gen_Tables where TABLE_NAME = 'CSA_TASKCENTRE_SERVERS')
begin
	insert Gen_Tables (TABLE_NAME, CODE, UPPER_CODE, SUBSTITUTE, DESCRIPTION)
		values ('CSA_TASKCENTRE_SERVERS', 'ServerAlias', 'SERVERALIAS', 'RealServerName;aTCLoginForCSA;TCPassword',
			'TaskCentre server for use by CSA. Code = nickname referenced by alert. Expansion ='
			+ ' server;user;password to be used by CSA separated by semicolons.')
end
go

print 'Creating configuration items (iPart version)'
go
if not exists (select * from SystemConfig where SystemConfigKey = '42FDD0CF-9892-4007-9FEC-B2E42FB0AFCF')
begin
	print '-- adding RSM.CSA.License'

	insert SystemConfigPageRef (SystemConfigPageKey, SystemConfigPageName, SystemConfigPageDesc)
		select '316F8E3A-8391-45E4-9005-BC5FDE08EFAC', 'RSMCustomSettings', 'Custom settings for McGladrey-created features (Variable names in parentheses)'
		where not exists (select * from SystemConfigPageRef where SystemConfigPageKey = '316F8E3A-8391-45E4-9005-BC5FDE08EFAC')

	exec asi_InsertSystemConfig 'Organization', 'RSMCustomSettings', 'MANAGER',
		'42FDD0CF-9892-4007-9FEC-B2E42FB0AFCF', 'RSM.CSA.License',
		'85F7051F-0D92-4802-A6CC-74E9B54EA825', 7, 6, 40,
		'License key for Customer Service Alerts iParts',
		'', 100
end
else
begin
	print '-- RSM.CSA.License already exists'
end
go


print 'Creating table tcsa_EventsForTaskCentre (used by iPart)'
go
if object_id('tcsa_EventsForTaskCentre') is not NULL
begin
	if (select rows from sysindexes where id = object_id('tcsa_EventsForTaskCentre') and indid < 2) > 0
	begin
		print '***Not deleting - data exists'
	end
	else
	begin
		drop table tcsa_EventsForTaskCentre
	end
end
go
if object_id('tcsa_EventsForTaskCentre') is NULL
begin
	create table tcsa_EventsForTaskCentre
	(
		-- Automatically assigned properties
		Seqn int identity,
		InsertedDate datetime not NULL default getdate(),
		InsertedBy nvarchar(128) not NULL default user_name(),
		OriginDatabase nvarchar(128) not NULL default db_name(),

		-- Populated by CSA
		ServerAlias varchar(60) not NULL,			-- Server alias, for compatibility with CSA desktop
		TaskId int not NULL,						-- Task which will handle this event
		UserName varchar(60) not NULL,				-- User who clicked the Action button
		SelectedId varchar(50) not NULL,			-- Contact to which the action applies
		Param1 nvarchar(255) not NULL default '',	-- Optional parameter 1
		Param2 nvarchar(255) not NULL default '',	-- Optional parameter 2

		-- Defaulted during insert, task may update
		ProcessedDate datetime NULL,
		ProcessedStatus int not NULL default 0,
		ProcessedMessage nvarchar(255) not NULL default ''
	)
end
go
if object_id('pkcsa_EventsForTaskCentre') is NULL
begin
	alter table tcsa_EventsForTaskCentre
		add constraint pkcsa_EventsForTaskCentre
		primary key clustered (Seqn)
end
go
if not exists (select * from sysindexes where id = object_id('tcsa_EventsForTaskCentre')
		and name = 'icsa_EventsForTaskCentreSelectedId')
begin
	create index icsa_EventsForTaskCentreSelectedId
		on tcsa_EventsForTaskCentre(SelectedId)
end
go
grant select, insert, update, delete, references, view definition on tcsa_EventsForTaskCentre to IMIS
grant select on tcsa_EventsForTaskCentre to public
go

print 'Creating table tcsa_ActionQueue (used by iPart)'
go
if object_id('tcsa_ActionQueue') is not NULL
begin
	-- Always ok to delete
	drop table tcsa_ActionQueue
end
go
create table tcsa_ActionQueue
(
	PageInstanceKey uniqueidentifier not NULL,
	ActionItemKey uniqueidentifier not NULL,
	CreatedDate datetime not NULL default getdate(),
	ExecuteCount int not NULL default 0,
	LastExecuted datetime NULL,

	ActionType int not NULL default 0,
	LoggedInId varchar(10) not NULL default '',
	SubjectId varchar(10) not NULL default '',
	RuleSeqn int not NULL default 0,
	CommandText nvarchar(4000) not NULL default N'',
	Seqn int identity
		constraint pkcsa_ActionQueue primary key clustered
)
go
create unique index icsa_ActionQueueKeys
	on tcsa_ActionQueue(PageInstanceKey, ActionItemKey)
go
grant select, insert, update, delete on tcsa_ActionQueue to iMIS
go

print 'Creating procedure pcsa_Internal_SaveAction'
go
if object_id('pcsa_Internal_SaveAction') is not NULL
begin
	drop procedure pcsa_Internal_SaveAction
end
go
create procedure pcsa_Internal_SaveAction
	@PageInstanceKey uniqueidentifier,
	@ActionItemKey uniqueidentifier,
	@ActionType int,
	@LoggedInId varchar(10),
	@SubjectId varchar(10),
	@RuleSeqn int,
	@CommandText nvarchar(4000)
as
/***************************************************************************************************
	© 2014 McGladrey LLP  All rights reserved.

McGladrey LLP (McGladrey) grants to Client a limited, non-exclusive, non-transferable right to
install and use the Software/Source Code solely for internal business purposes.  Client may not
distribute, give, rent, loan, or resell a copy of the Software/Source Code to any third party.
McGladrey is the owner of the copyright for the Software/Source Code and McGladrey reserves all
rights related to such ownership.  Any Warranty granted by McGladrey is void if Software/Source Code
is modified by anyone other than McGladrey.

$URL: https://svn.mcgladreycloud.com/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 7851 $
$Date: 2014-02-03 15:48:42 -0600 (Mon, 03 Feb 2014) $
***************************************************************************************************/
-- Insert the provided action details to the action queue
begin
	set nocount on

	insert tcsa_ActionQueue (PageInstanceKey, ActionItemKey, ActionType, LoggedInId, SubjectId, RuleSeqn, CommandText)
		values (@PageInstanceKey, @ActionItemKey, @ActionType, @LoggedInId, @SubjectId, @RuleSeqn, @CommandText)
end
go
grant execute, view definition on pcsa_Internal_SaveAction to IMIS
go

print 'Creating procedure pcsa_Internal_GroomActions'
go
if object_id('pcsa_Internal_GroomActions') is not NULL
begin
	drop procedure pcsa_Internal_GroomActions
end
go
create procedure pcsa_Internal_GroomActions
	@numDays int = 30,
	@includeExecuted bit = 0
as
/***************************************************************************************************
	© 2014 McGladrey LLP  All rights reserved.

McGladrey LLP (McGladrey) grants to Client a limited, non-exclusive, non-transferable right to
install and use the Software/Source Code solely for internal business purposes.  Client may not
distribute, give, rent, loan, or resell a copy of the Software/Source Code to any third party.
McGladrey is the owner of the copyright for the Software/Source Code and McGladrey reserves all
rights related to such ownership.  Any Warranty granted by McGladrey is void if Software/Source Code
is modified by anyone other than McGladrey.

$URL: https://svn.mcgladreycloud.com/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 7851 $
$Date: 2014-02-03 15:48:42 -0600 (Mon, 03 Feb 2014) $
***************************************************************************************************/
-- Delete actions older than N days. By default, 30 days, only those never executed.
begin
	set nocount on

	declare @since datetime

	set @since = dateadd(day, -1 * abs(@numDays), getdate())

	if @includeExecuted = 1
	begin
		delete tcsa_ActionQueue
			where CreatedDate < @since
	end
	else
	begin
		delete tcsa_ActionQueue
			where CreatedDate < @since
				and ExecuteCount = 0
	end

end
go
grant execute, view definition on pcsa_Internal_GroomActions to IMIS
go


print 'Creating procedure pcsa_Internal_ExecuteAction'
go
if object_id('pcsa_Internal_ExecuteAction') is not NULL
begin
	drop procedure pcsa_Internal_ExecuteAction
end
go
create procedure pcsa_Internal_ExecuteAction
	@PageInstanceKey uniqueidentifier,
	@ActionItemKey uniqueidentifier
as
/***************************************************************************************************
	© 2014 McGladrey LLP  All rights reserved.

McGladrey LLP (McGladrey) grants to Client a limited, non-exclusive, non-transferable right to
install and use the Software/Source Code solely for internal business purposes.  Client may not
distribute, give, rent, loan, or resell a copy of the Software/Source Code to any third party.
McGladrey is the owner of the copyright for the Software/Source Code and McGladrey reserves all
rights related to such ownership.  Any Warranty granted by McGladrey is void if Software/Source Code
is modified by anyone other than McGladrey.

$URL: https://svn.mcgladreycloud.com/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 7851 $
$Date: 2014-02-03 15:48:42 -0600 (Mon, 03 Feb 2014) $
***************************************************************************************************/
-- Update the queue, execute the action directly if SQL or TC, otherwise return redirection URL to caller
begin
	set nocount on

	declare @actionType int,
		@url nvarchar(255),
		@message nvarchar(255),
		@loggedInId varchar(10),
		@subjectId varchar(10),
		@commandText nvarchar(4000)

	set @message = ''

	update aq
		set ExecuteCount = ExecuteCount + 1,
			LastExecuted = getdate(),
			@actionType = ActionType,
			@loggedInId = LoggedInId,
			@subjectId = SubjectId,
			@commandText = CommandText
		from tcsa_ActionQueue aq
		where aq.PageInstanceKey = @pageInstanceKey
			and aq.ActionItemKey = @actionItemKey

	if @@rowcount != 1
	begin
		set @actionType = 0
		set @message = 'No matching action found'
		goto done
	end

	exec pcsa_Internal_GroomActions

done:
	select @message 'ReturnMessage', @actionType 'ActionType', @loggedInId 'LoggedInId', @subjectId 'SubjectId', @commandText 'CommandText'
end
go
grant execute, view definition on pcsa_Internal_ExecuteAction to IMIS
go

print 'Finished.'